#!/bin/zsh
# Script to scavenge LVM2 device definitions.

# Copyright (C) 2009, David W. Noon.  All rights reserved.
# This code is released under the GNU General Public License (GPL), any version.

function byte_size
{
	local -r suffix_list="E P T G M K"
	local suffix
	local -i denom=$((1<<60)) rem

	for suffix in ${=suffix_list}
	do
		let rem=${1}%denom
		if [[ $rem -eq 0 ]]; then
			echo "$((${1}/denom))$suffix"
			return 0
		fi
		let denom=$((denom>>10))
	done

	echo "${1}B"
	return 0
}

function lvm_rebuild
{
	local dm_device final_name vol_group vol_name label_option fs_type
	local -i blksize vol_size

	echo "#!/bin/sh"

	for dm_device in /sys/devices/virtual/block/dm-*
	do
		final_name=$(cat "$dm_device/dm/name")
		vol_group=${final_name%%-*}
		vol_name=${final_name##*-}

		# Is this right??
		if [ -f "$dm_device/dm/label" ]; then
			label_option="-L '$(cat $dm_device/dm/label)' "
		else
			label_option=''
		fi

		let blksize=$(cat "$dm_device/queue/logical_block_size")
		let vol_size=$(cat "$dm_device/size")

		echo "lvcreate -L $(byte_size $((vol_size*blksize))) -n $vol_name $vol_group"

		# Sort out the filesystem type, ready for mkfs.
		fs_type=$(gawk "/^\/dev\/mapper\/$final_name / { print \$3; }" /etc/mtab)
		if [ -z "$fs_type" ]; then
			if [[ $UID -eq 0 ]]; then
				fs_type=$(/sbin/blkid -o udev -p /dev/mapper/$final_name | gawk -F '=' '/^ID_FS_TYPE=/ { print $2; }')
			else
				fs_type=$(sudo /sbin/blkid -o udev -p /dev/mapper/$final_name | gawk -F '=' '/^ID_FS_TYPE=/ { print $2; }')
			fi
		fi
		case "$fs_type" in
			ext2)
				echo "mke2fs ${label_option}/dev/$vol_group/$vol_name"
				;;

			ext[34])
				echo "mkfs.$fs_type -j -O dir_index ${label_option}/dev/$vol_group/$vol_name"
				;;

			reiserfs)
				# ReiserFS uses a different label option from ext2/3/4.
				echo "mkreiserfs ${label_option/-L/-l}/dev/$vol_group/$vol_name"
				;;

			swap)
				echo "mkswap ${label_option}/dev/$vol_group/$vol_name"
				;;

			*)
				echo "mkfs.$fs_type ${label_option}/dev/$vol_group/$vol_name"
				;;
		esac
	done

	return 0
}

lvm_rebuild | sort

exit 0
